using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using Syncfusion.ComponentModel;

namespace Syncfusion.Windows.Forms.Grid
{

	#region CustomFindReplaceDialog

	/// <summary>
	/// A find and replace dialog that can be used to search cells within a grid. The dialog
	/// communicates with the grid through the <see cref="IGridFindReplaceDialogSink"/> interface.
	/// </summary>
	/// <remarks>
	/// See the  <see cref="CustomFindReplaceDialogSink"/> for an example of how to enable find and
	/// replace support for a grid.
	/// <para/>
	/// If you want to find and replace in the grid without displaying this dialog,
	/// you should instantiate a <see cref="CustomFindReplaceDialogSink"/> object and call its
	/// <see cref="CustomFindReplaceDialogSink.Find"/> or <see cref="CustomFindReplaceDialogSink.Replace"/> \
	/// method. <para/>
	/// </remarks>
	public class CustomFindReplaceDialog 
		: Form
	{
		private IGridFindReplaceDialogSink editView;
		private object lcrFoundOld;
		private int editOriginalCloseTop;
		private int offsetY;
		//		private bool bFound = false;
		private bool inFind = false;
		private System.Windows.Forms.Label labelFindWhat;
		private System.Windows.Forms.CheckBox checkBoxMatchCase;
		private System.Windows.Forms.CheckBox checkBoxMatchWholeCell;
		private System.Windows.Forms.CheckBox checkBoxSearchUp;
		private System.Windows.Forms.Button buttonReplace;
		private System.Windows.Forms.Button buttonClose;
		private System.Windows.Forms.Button buttonFindNext;
		private System.Windows.Forms.Button buttonReplaceAll;
		private System.Windows.Forms.Label labelReplaceWith;
		private System.Windows.Forms.ComboBox comboBoxFindWhat;
		private System.Windows.Forms.ComboBox comboBoxReplaceWith;
		private System.Windows.Forms.ImageList imageList1;
		private System.Windows.Forms.ComboBox comboBoxSelection;
		private System.ComponentModel.IContainer components;

		const int indexWholeTable = 0;
		const int indexColumnOnly = 1;
		const int indexSelectionOnly = 2;

		// TODO: Add support for localization
		string strNotFound = "The specified text was not found.";
		string strCategory = "Search Results";
		string strReachedStart = "Find reached the starting point of the search.";

		[ThreadStaticAttribute] static CustomFindReplaceDialog dialog = null;

		/// <summary>
		/// Returns an instance of the find replace dialog. Creates a single
		/// shared instance if necessary.
		/// </summary>
		public static CustomFindReplaceDialog Instance
		{
			get
			{
				if (dialog == null)
					dialog = new CustomFindReplaceDialog();
				return dialog;
			}
		}

		/// <summary>
		/// Disallow explicit construction of this dialog. Use <see cref="Instance"/>
		/// instead.
		/// </summary>
		CustomFindReplaceDialog()
		{
			InitializeComponent();

			editOriginalCloseTop = buttonClose.Top;
			offsetY = editOriginalCloseTop - buttonReplaceAll.Top;
			this.comboBoxSelection.SelectedIndex = indexWholeTable;//indexColumnOnly;
			ApplyOptions();
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		/// <summary>
		/// The text to be searched.
		/// </summary>
		public string TextSearched
		{
			get
			{
				return comboBoxFindWhat.Text;
			}
			set
			{
				comboBoxFindWhat.Text = value;
			}
		}

		CustomFindReplaceDialogOptions options = CustomFindReplaceDialogOptions.All;

		/// <summary>
		/// Show or hide various dialog elements.
		/// </summary>
		[DefaultValue(CustomFindReplaceDialogOptions.All)]
		public CustomFindReplaceDialogOptions Options
		{
			get
			{
				return this.options;
			}
			set
			{
				if (this.options != value)
				{
					this.options = value;
					OnOptionsChanged(EventArgs.Empty);
					ApplyOptions();
				}
			}
		}

		/// <summary>
		/// Occurs when the <see cref="Options"/> property has changed.
		/// </summary>
		public event EventHandler OptionsChanged;

		/// <summary>
		/// Raises the <see cref="OptionsChanged"/> event.
		/// </summary>
		/// <param name="e">A <see cref="EventArgs" /> that contains the event data.</param>
		protected virtual void OnOptionsChanged(EventArgs e)
		{
			if (OptionsChanged != null)
				OptionsChanged(this, e);
		}

		/// <summary>
		/// Register the current target of find and replace operations. Call this
		/// from your controls <see cref="Control.Enter"/> event handler.
		/// </summary>
		/// <param name="sink">The new target for find and replace operations.</param>
		/// <remarks>
		/// The method checks if there is an active dialog displayed. Otherwise,
		/// the method will have no effect.
		/// </remarks>
		public static void SetActiveSinkIfVisible(IGridFindReplaceDialogSink sink)
		{
			if (dialog != null)
				dialog.ActiveSink = sink;
		}

		/// <summary>
		/// Set current selection in your current find and replace target control. For a 
		/// grid, this should be a GridRangeInfo.
		/// </summary>
		public static void ResetFindLocation()
		{
			if (dialog != null && !dialog.inFind)
				dialog.lcrFoundOld = null;
		}

		/// <summary>
		/// Returns the current target of find and replace operations.
		/// </summary>
		public IGridFindReplaceDialogSink ActiveSink
		{
			get
			{
				return editView;
			}
			set
			{
				editView = value;
			}
		}

		/// <summary>
		/// Adds the string to the search list of the find what combo box.
		/// </summary>
		/// <param name="str">The search string to be added.</param>
		public void AddToSearchedList(string str)
		{
			if (!comboBoxFindWhat.Items.Contains(comboBoxFindWhat.Text))
			{
				comboBoxFindWhat.Items.Insert(0, comboBoxFindWhat.Text);
			}
		}

		/// <override/>
		protected override void OnHandleDestroyed(EventArgs e)
		{
			dialog = null;
			base.OnHandleDestroyed (e);
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(CustomFindReplaceDialog));
			this.buttonReplaceAll = new System.Windows.Forms.Button();
			this.labelReplaceWith = new System.Windows.Forms.Label();
			this.checkBoxSearchUp = new System.Windows.Forms.CheckBox();
			this.comboBoxReplaceWith = new System.Windows.Forms.ComboBox();
			this.imageList1 = new System.Windows.Forms.ImageList(this.components);
			this.checkBoxMatchCase = new System.Windows.Forms.CheckBox();
			this.labelFindWhat = new System.Windows.Forms.Label();
			this.comboBoxFindWhat = new System.Windows.Forms.ComboBox();
			this.checkBoxMatchWholeCell = new System.Windows.Forms.CheckBox();
			this.buttonClose = new System.Windows.Forms.Button();
			this.buttonReplace = new System.Windows.Forms.Button();
			this.buttonFindNext = new System.Windows.Forms.Button();
			this.comboBoxSelection = new System.Windows.Forms.ComboBox();
			this.SuspendLayout();
			// 
			// buttonReplaceAll
			// 
			this.buttonReplaceAll.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.buttonReplaceAll.Location = new System.Drawing.Point(352, 64);
			this.buttonReplaceAll.Name = "buttonReplaceAll";
			this.buttonReplaceAll.TabIndex = 16;
			this.buttonReplaceAll.Text = "Replace &All";
			this.buttonReplaceAll.Click += new System.EventHandler(this.buttonReplaceAll_Click);
			// 
			// labelReplaceWith
			// 
			this.labelReplaceWith.Location = new System.Drawing.Point(8, 36);
			this.labelReplaceWith.Name = "labelReplaceWith";
			this.labelReplaceWith.Size = new System.Drawing.Size(84, 14);
			this.labelReplaceWith.TabIndex = 3;
			this.labelReplaceWith.Text = "Re&place with:";
			// 
			// checkBoxSearchUp
			// 
			this.checkBoxSearchUp.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.checkBoxSearchUp.Location = new System.Drawing.Point(208, 64);
			this.checkBoxSearchUp.Name = "checkBoxSearchUp";
			this.checkBoxSearchUp.Size = new System.Drawing.Size(124, 16);
			this.checkBoxSearchUp.TabIndex = 10;
			this.checkBoxSearchUp.Text = "Search &up";
			// 
			// comboBoxReplaceWith
			// 
			this.comboBoxReplaceWith.DropDownWidth = 280;
			this.comboBoxReplaceWith.Location = new System.Drawing.Point(96, 32);
			this.comboBoxReplaceWith.Name = "comboBoxReplaceWith";
			this.comboBoxReplaceWith.Size = new System.Drawing.Size(240, 21);
			this.comboBoxReplaceWith.TabIndex = 4;
			// 
			// imageList1
			// 
			this.imageList1.ImageSize = new System.Drawing.Size(16, 16);
			this.imageList1.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList1.ImageStream")));
			this.imageList1.TransparentColor = System.Drawing.Color.White;
			// 
			// checkBoxMatchCase
			// 
			this.checkBoxMatchCase.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.checkBoxMatchCase.Location = new System.Drawing.Point(8, 64);
			this.checkBoxMatchCase.Name = "checkBoxMatchCase";
			this.checkBoxMatchCase.Size = new System.Drawing.Size(124, 16);
			this.checkBoxMatchCase.TabIndex = 6;
			this.checkBoxMatchCase.Text = "Match &case";
			// 
			// labelFindWhat
			// 
			this.labelFindWhat.Location = new System.Drawing.Point(8, 8);
			this.labelFindWhat.Name = "labelFindWhat";
			this.labelFindWhat.Size = new System.Drawing.Size(84, 14);
			this.labelFindWhat.TabIndex = 0;
			this.labelFindWhat.Text = "Fi&nd what:";
			// 
			// comboBoxFindWhat
			// 
			this.comboBoxFindWhat.DropDownWidth = 256;
			this.comboBoxFindWhat.Location = new System.Drawing.Point(96, 4);
			this.comboBoxFindWhat.Name = "comboBoxFindWhat";
			this.comboBoxFindWhat.Size = new System.Drawing.Size(240, 21);
			this.comboBoxFindWhat.TabIndex = 1;
			this.comboBoxFindWhat.TextChanged += new System.EventHandler(this.comboBoxFindWhat_TextChanged);
			// 
			// checkBoxMatchWholeCell
			// 
			this.checkBoxMatchWholeCell.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.checkBoxMatchWholeCell.Location = new System.Drawing.Point(8, 94);
			this.checkBoxMatchWholeCell.Name = "checkBoxMatchWholeCell";
			this.checkBoxMatchWholeCell.Size = new System.Drawing.Size(124, 16);
			this.checkBoxMatchWholeCell.TabIndex = 7;
			this.checkBoxMatchWholeCell.Text = "Match &whole cell";
			// 
			// buttonClose
			// 
			this.buttonClose.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.buttonClose.Location = new System.Drawing.Point(352, 92);
			this.buttonClose.Name = "buttonClose";
			this.buttonClose.TabIndex = 18;
			this.buttonClose.Text = "Close";
			this.buttonClose.Click += new System.EventHandler(this.buttonClose_Click);
			// 
			// buttonReplace
			// 
			this.buttonReplace.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.buttonReplace.ImageAlign = System.Drawing.ContentAlignment.MiddleRight;
			this.buttonReplace.ImageIndex = 1;
			this.buttonReplace.ImageList = this.imageList1;
			this.buttonReplace.Location = new System.Drawing.Point(352, 36);
			this.buttonReplace.Name = "buttonReplace";
			this.buttonReplace.TabIndex = 15;
			this.buttonReplace.Text = "&Replace";
			this.buttonReplace.Click += new System.EventHandler(this.buttonReplace_Click);
			// 
			// buttonFindNext
			// 
			this.buttonFindNext.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.buttonFindNext.Location = new System.Drawing.Point(352, 8);
			this.buttonFindNext.Name = "buttonFindNext";
			this.buttonFindNext.TabIndex = 14;
			this.buttonFindNext.Text = "&Find Next";
			this.buttonFindNext.Click += new System.EventHandler(this.buttonFindNext_Click);
			// 
			// comboBoxSelection
			// 
			this.comboBoxSelection.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboBoxSelection.DropDownWidth = 128;
			this.comboBoxSelection.Enabled = false;
			this.comboBoxSelection.Items.AddRange(new object[] {
																   "Whole table",
																   "Column only",
																   "Selection only"});
			this.comboBoxSelection.Location = new System.Drawing.Point(208, 92);
			this.comboBoxSelection.Name = "comboBoxSelection";
			this.comboBoxSelection.Size = new System.Drawing.Size(128, 21);
			this.comboBoxSelection.TabIndex = 13;
			// 
			// CustomFindReplaceDialog
			// 
			this.AcceptButton = this.buttonFindNext;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(434, 124);
			this.Controls.Add(this.comboBoxReplaceWith);
			this.Controls.Add(this.labelReplaceWith);
			this.Controls.Add(this.buttonReplaceAll);
			this.Controls.Add(this.buttonClose);
			this.Controls.Add(this.buttonReplace);
			this.Controls.Add(this.buttonFindNext);
			this.Controls.Add(this.comboBoxFindWhat);
			this.Controls.Add(this.checkBoxSearchUp);
			this.Controls.Add(this.checkBoxMatchWholeCell);
			this.Controls.Add(this.checkBoxMatchCase);
			this.Controls.Add(this.labelFindWhat);
			this.Controls.Add(this.comboBoxSelection);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "CustomFindReplaceDialog";
			this.Text = "CustomFindReplaceDialog";
			this.TopMost = true;
			this.Closing += new System.ComponentModel.CancelEventHandler(this.CustomFindReplaceDialog_Closing);
			this.VisibleChanged += new System.EventHandler(this.This_VisibleChanged);
			this.ResumeLayout(false);

		}
		#endregion

		private void This_VisibleChanged(object sender, System.EventArgs e)
		{
			if (this.Visible)
			{
				// Make "Find What" the focused control.
				comboBoxFindWhat.Select();
			}
			else
			{
				if (this.editView is Control)
					((Control) this.editView).Focus();
			}
		}

		private void buttonClose_Click(object sender, System.EventArgs e)
		{
			this.Hide();
		}

		/// <summary>
		/// Occurs when the find text was not found.
		/// </summary>
		/// <remarks>
		/// Set e.Cancel = True if you do not want to display the
		/// default "not found" message box or if you want to display
		/// a customized dialog.
		/// </remarks>
		public event CancelEventHandler TextNotFound;

		/// <summary>
		/// Raises the <see cref="TextNotFound"/> event.
		/// </summary>
		/// <param name="e">A <see cref="CancelEventArgs" /> that contains the event data.</param>
		protected virtual void OnTextNotFound(CancelEventArgs e)
		{
			if (TextNotFound != null)
				TextNotFound(this, e);

			if (!e.Cancel)
			{
				this.buttonClose_Click(this, e);
				editView.ShowInfoMessage(strNotFound, strCategory);
			}
		}

		/// <summary>
		/// Occurs when the find operation reaches the starting point again.
		/// </summary>
		/// <remarks>
		/// Set e.Cancel = True if you do not want to display the
		/// default "reached starting point" message box or if you want to display
		/// a customized dialog.
		/// </remarks>
		public event CancelEventHandler ReachedStartingPoint;

		/// <summary>
		/// Raises the <see cref="ReachedStartingPoint"/> event.
		/// </summary>
		/// <param name="e">A <see cref="CancelEventArgs" /> that contains the event data.</param>
		protected virtual void OnReachedStartingPoint(CancelEventArgs e)
		{
			if (ReachedStartingPoint != null)
				ReachedStartingPoint(this, e);

			if (!e.Cancel)
			{
				this.buttonClose_Click(this, e);
				editView.ShowInfoMessage(strReachedStart, strCategory);
			}
		}


		/// <summary>
		/// Find Next occurrance of string.
		/// </summary>
		public void FindNext()
		{
			buttonFindNext_Click(this, System.EventArgs.Empty);
		}

		private void buttonFindNext_Click(object sender, System.EventArgs e)
		{
			object foundOld = this.lcrFoundOld;
			inFind = true;

			try
			{
				if (editView != null)
				{
					AddToSearchedList(comboBoxFindWhat.Text);
					GridFindReplaceEventArgs fea = CreateFindReplaceEventArgs();
					object lcrFound = editView.Find(fea);
					if (lcrFound == null)
					{
						OnTextNotFound(new CancelEventArgs(false));
					}
					else
					{
						Rectangle bounds = editView.GetScreenRect(lcrFound);
						Point ptTemp = new Point(bounds.Right, bounds.Bottom);
						if (this.Bounds.Contains(ptTemp))
						{
							if (Screen.PrimaryScreen.WorkingArea.Contains(new 
								Rectangle(ptTemp.X + 8, this.Top, this.Width, this.Height)))
							{
								this.Left = ptTemp.X + 8;
							}
							else if (Screen.PrimaryScreen.WorkingArea.Contains(new 
								Rectangle(this.Left, ptTemp.Y - 8 - this.Height, 
								this.Width, this.Height)))
							{
								this.Top = ptTemp.Y - 8 - this.Height;
							}
							else 
							{
								ptTemp = bounds.Location;
								if (Screen.PrimaryScreen.WorkingArea.Contains(new 
									Rectangle(ptTemp.X - 8 - this.Width, this.Top, 
									this.Width, this.Height)))
								{
									this.Left = ptTemp.X - 8 - this.Width;
								}
								else
								{
									this.Left = 1;
									this.Top = 1;
								}
							}
						}
						if (lcrFound.Equals(foundOld))
						{
							OnReachedStartingPoint(new CancelEventArgs(false));
						}
						else
						{
							if (lcrFoundOld == null)
							{
								lcrFoundOld = lcrFound;
							}
						}
						//						bFound = true;
					}
				}
			}
			finally
			{
				inFind = false;
			}
		}
		//
		//		private void comboBoxFindWhat_KeyPress(object sender, System.Windows.Forms.KeyPressEventArgs e)
		//		{
		//			if (e.KeyChar == '\r')
		//			{
		//				if (editView != null)
		//				{
		//					GridFindReplaceEventArgs fea = CreateFindReplaceEventArgs();
		//					if (!editView.FindNext(fea))
		//					{
		//						editView.ShowInfoMessage("The specified text was not found.", 
		//							"Search Results");
		//					}
		//				}
		//			}
		//		}

		private void buttonReplace_Click(object sender, System.EventArgs e)
		{
			if (buttonClose.Top != editOriginalCloseTop)
			{
				SetToReplaceState();
				this.comboBoxReplaceWith.Focus();
			}
			else
			{
				object foundOld = this.lcrFoundOld;
				inFind = true;

				try
				{
					if (editView != null)
					{
						AddToSearchedList(comboBoxFindWhat.Text);
						GridFindReplaceEventArgs fea = CreateFindReplaceEventArgs();
						object lcrFound = editView.Replace(fea);
						if (lcrFound == null)
						{
							OnTextNotFound(new CancelEventArgs(false));
						}
						else
						{
							Rectangle bounds = editView.GetScreenRect(lcrFound);
							Point ptTemp = new Point(bounds.Right, bounds.Bottom);
							if (this.Bounds.Contains(ptTemp))
							{
								if (Screen.PrimaryScreen.WorkingArea.Contains(new 
									Rectangle(ptTemp.X + 8, this.Top, this.Width, this.Height)))
								{
									this.Left = ptTemp.X + 8;
								}
								else if (Screen.PrimaryScreen.WorkingArea.Contains(new 
									Rectangle(this.Left, ptTemp.Y - 8 - this.Height, 
									this.Width, this.Height)))
								{
									this.Top = ptTemp.Y - 8 - this.Height;
								}
								else 
								{
									ptTemp = bounds.Location;
									if (Screen.PrimaryScreen.WorkingArea.Contains(new 
										Rectangle(ptTemp.X - 8 - this.Width, this.Top, 
										this.Width, this.Height)))
									{
										this.Left = ptTemp.X - 8 - this.Width;
									}
									else
									{
										this.Left = 1;
										this.Top = 1;
									}
								}
							}
							if (lcrFound.Equals(foundOld))
							{
								OnReachedStartingPoint(new CancelEventArgs(false));
							}
							else
							{
								if (lcrFoundOld == null)
								{
									lcrFoundOld = lcrFound;
								}
							}
							//							bFound = true;
						}
					}
				}
				finally
				{
					inFind = false;
				}
			}
		}


		internal static GridFindTextOptions MakeFindOptions(bool bMatchCase, 
			bool bMatchWholeCell, bool bSearchUp, 
			bool bWholeTable, bool bColumnOnly, bool bSelectionOnly)
		{
			GridFindTextOptions opt = GridFindTextOptions.None;
			if (bMatchCase)
				opt |= GridFindTextOptions.MatchCase;
			if (bMatchWholeCell)
				opt |= GridFindTextOptions.MatchWholeCell;
			if (bSearchUp)
				opt |= GridFindTextOptions.SearchUp;
			if (bSelectionOnly)
				opt |= GridFindTextOptions.SelectionOnly;
			if (bColumnOnly)
				opt |= GridFindTextOptions.ColumnOnly;
			if (bWholeTable)
				opt |= GridFindTextOptions.WholeTable;
			return opt;
		}

		private GridFindReplaceEventArgs CreateFindReplaceEventArgs()
		{
			bool wholeTable = this.comboBoxSelection.SelectedIndex == indexWholeTable;
			bool columnOnly = this.comboBoxSelection.SelectedIndex == indexColumnOnly;
			bool selectionOnly = this.comboBoxSelection.SelectedIndex == indexSelectionOnly;

			GridFindReplaceEventArgs fea = new GridFindReplaceEventArgs(
				comboBoxFindWhat.Text, comboBoxReplaceWith.Text, 
				MakeFindOptions(
				checkBoxMatchCase.Checked, checkBoxMatchWholeCell.Checked, 
				checkBoxSearchUp.Checked, 
				wholeTable, columnOnly, selectionOnly),
				lcrFoundOld);

			return fea;
		}

		private void comboBoxFindWhat_TextChanged(object sender, System.EventArgs e)
		{
			CheckButtonEnabledState();
			lcrFoundOld = null;
		}

		internal void SetFindText(string str)
		{
			comboBoxFindWhat.Text = str;
		}

		/// <summary>
		/// Initializes the dialog.
		/// </summary>
		/// <param name="view">The target for find / replace operations.</param>
		/// <param name="str">The text to be shown in find text box.</param>
		/// <param name="bReplace">Specifies if dialog should be shown in find-only or replace mode.</param>
		public void SetState(IGridFindReplaceDialogSink view, string str, bool bReplace)
		{
			this.editView = view;
			comboBoxFindWhat.Text = str;
			//			bFound = false;
			if (bReplace)
			{
				SetToReplaceState();
			}
			else
			{
				SetToFindState();
			}
		}

		/// <summary>
		/// Gets or sets whether the "Replace" button should be visible. Set this false
		/// if you called <see cref="SetState"/> to show dialog in find-only mode and do not
		/// want users to give a choice to switch to replace mode.
		/// </summary>
		public bool ShowReplaceButton
		{
			get
			{
				return buttonReplaceAll.Visible;
			}
			set
			{
				buttonReplaceAll.Visible = false;
			}
		}

		private void SetToFindState()
		{
			if (buttonClose.Top == editOriginalCloseTop)
			{
				this.Height -= offsetY;
				labelReplaceWith.Visible = false;
				comboBoxReplaceWith.Visible = false;
				buttonReplaceAll.Visible = false;
				buttonReplace.ImageIndex = 1;
				buttonClose.Top -= offsetY;
				checkBoxMatchCase.Top -= offsetY;
				checkBoxMatchWholeCell.Top -= offsetY;
				checkBoxSearchUp.Top -= offsetY;
				comboBoxSelection.Top -= offsetY;
				CheckButtonEnabledState();
			}
		}

		private void SetToReplaceState()
		{
			if (buttonClose.Top != editOriginalCloseTop)
			{
				this.Height += offsetY;
				labelReplaceWith.Visible = true;
				comboBoxReplaceWith.Visible = true;
				buttonReplaceAll.Visible = true;
				buttonReplace.ImageIndex = -1;
				buttonClose.Top += offsetY;
				checkBoxMatchCase.Top += offsetY;
				checkBoxMatchWholeCell.Top += offsetY;
				checkBoxSearchUp.Top += offsetY;
				comboBoxSelection.Top += offsetY;
				if (GridUtil.IsEmpty(comboBoxFindWhat.Text))
				{
					buttonReplace.Enabled = false;
				}
				comboBoxFindWhat.Focus();
			}
		}

		private void CheckButtonEnabledState()
		{
			if (GridUtil.IsEmpty(comboBoxFindWhat.Text))
			{
				buttonFindNext.Enabled = false;
				if (buttonClose.Top == editOriginalCloseTop)
				{
					buttonReplace.Enabled = false;
				}
				else
				{
					buttonReplace.Enabled = true;
				}
				buttonReplaceAll.Enabled = false;
			}
			else
			{
				buttonFindNext.Enabled = true;
				buttonReplace.Enabled = true;
				buttonReplaceAll.Enabled = true;
			}
		}

		private void buttonReplaceAll_Click(object sender, System.EventArgs e)
		{
			GridFindReplaceEventArgs fea = CreateFindReplaceEventArgs();
			editView.ReplaceAll(fea);
		}

		private string GetReplacedString(string strOld)
		{
			return comboBoxReplaceWith.Text;
		}

		private void CustomFindReplaceDialog_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			this.Hide();
			e.Cancel = true;
		}

		void ApplyOptions()
		{
			if ((options & CustomFindReplaceDialogOptions.ShowMatchCase) != CustomFindReplaceDialogOptions.None)
			{
				this.checkBoxMatchCase.Visible = true;
			}
			else
			{
				this.checkBoxMatchCase.Visible = false;
			}

			if ((options & CustomFindReplaceDialogOptions.ShowMatchWholeCell) != CustomFindReplaceDialogOptions.None)
			{
				this.checkBoxMatchWholeCell.Visible = true;
			}
			else
			{
				this.checkBoxMatchWholeCell.Visible = false;
			}

			if ((options & CustomFindReplaceDialogOptions.ShowSearchSelection) != CustomFindReplaceDialogOptions.None)
			{
				this.comboBoxSelection.Visible = true;
				this.comboBoxSelection.Enabled = true;
			}
			else
			{
				this.comboBoxSelection.Visible = false;
			}

			if ((options & CustomFindReplaceDialogOptions.ShowSearchUp) != CustomFindReplaceDialogOptions.None)
			{
				this.checkBoxSearchUp.Visible = true;
			}
			else
			{
				this.checkBoxSearchUp.Visible = false;
			}
		}
	}


	/// <summary>
	/// Options used for customizing <see cref="CustomFindReplaceDialog"/>:
	/// </summary>
	[Flags]
	public enum CustomFindReplaceDialogOptions
	{
		/// <summary>
		/// None.
		/// </summary>
		None					= 0,
		/// <summary>
		/// Show Match Case check box.
		/// </summary>
		ShowMatchCase			= 1,
		/// <summary>
		/// Show Match Case check box.
		/// </summary>
		ShowMatchWholeCell		= 2,
		/// <summary>
		/// Show Search Up check box.
		/// </summary>
		ShowSearchUp			= 4,
		/// <summary>
		/// Show Selection combo box.
		/// </summary>
		ShowSearchSelection		= 8,

		/// <summary>
		/// Default setting.
		/// </summary>
		[Browsable(false)]
		All						= 15
	}

	// Should be used as input for Find, Replace etc.

	// CustomFindReplaceDialogSink vs DataBoundCustomFindReplaceDialogSink

	/// <summary>
	/// Provides a default implementation of a <see cref="IGridFindReplaceDialogSink"/>
	/// for a grid control.
	/// </summary>
	/// <example>
	/// The following example demonstrates how to enable a grid control for find and replace support:
	/// <code lang="C#">
	/// CustomFindReplaceDialogSink findReplaceDialogSink;
	/// 
	/// public CustomFindReplaceDialogSink CustomFindReplaceDialogSink
	/// {
	///     get
	///     {
	///         if (findReplaceDialogSink == null)
	///         {
	///             findReplaceDialogSink = new CustomFindReplaceDialogSink(this);
	///         }
	///         return findReplaceDialogSink;
	///     }
	/// }
	/// 
	/// protected override void OnCurrentCellActivated(EventArgs e)
	/// {
	///     CustomFindReplaceDialog.ResetFindLocation();
	///     base.OnCurrentCellActivated(e);
	/// }
	/// 
	/// protected override void OnControlGotFocus()
	/// {
	///     CustomFindReplaceDialog.SetActiveSinkIfVisible(CustomFindReplaceDialogSink);
	///     base.OnControlGotFocus();
	/// }
	/// 
	/// // to show the dialog:
	/// CustomFindReplaceDialog frDialog = CustomFindReplaceDialog.Instance;
	/// frDialog.SetState(grid.CustomFindReplaceDialogSink, "", false);
	/// frDialog.Show();
	/// </code>
	/// <code lang="VB">
	/// Dim findReplaceDialogSink As CustomFindReplaceDialogSink
	/// 
	/// 
	/// Public ReadOnly Property CustomFindReplaceDialogSink() As CustomFindReplaceDialogSink
	///     Get
	///         If findReplaceDialogSink Is Nothing Then
	///             findReplaceDialogSink = New CustomFindReplaceDialogSink(Me)
	///         End If
	///         Return findReplaceDialogSink
	///     End Get
	/// End Property
	/// 
	/// 
	/// Protected Overrides Sub OnCurrentCellActivated(e As EventArgs)
	///     CustomFindReplaceDialog.ResetFindLocation()
	///     MyBase.OnCurrentCellActivated(e)
	/// End Sub 'OnCurrentCellActivated
	/// 
	/// 
	/// Protected Overrides Sub OnControlGotFocus()
	///     CustomFindReplaceDialog.SetActiveSinkIfVisible(CustomFindReplaceDialogSink)
	///     MyBase.OnControlGotFocus()
	/// End Sub 'OnEnter
	/// 
	/// ' to show the dialog:
	/// Dim frDialog As CustomFindReplaceDialog = CustomFindReplaceDialog.Instance
	/// frDialog.SetState(grid.CustomFindReplaceDialogSink, "", False)
	/// frDialog.Show()
	/// </code>
	/// </example>





	#endregion


	public class CustomFindReplaceDialogSink : GridSubComponent, IGridFindReplaceDialogSink
	{
		enum Operation
		{
			None,
			Find,
			Replace,
			ReplaceAll
		}

		Operation op = Operation.None;
		bool replaceAllFound = false;

		/// <summary>
		/// Constructs a CustomFindReplaceDialogSink and associates it with a grid control.
		/// </summary>
		/// <param name="grid">The grid control this object operates on.</param>
		public CustomFindReplaceDialogSink(GridControlBase grid)
			: base(grid)
		{
		}

		/// <summary>
		/// Finds the text with the specified criteria.
		/// </summary>
		/// <param name="e">Text and search criteria.</param>
		/// <returns>The location info (typically a <see cref="GridRangeInfo"/>) or NULL if not found.</returns>
		public virtual object Find(GridFindReplaceEventArgs e)
		{
			op = Operation.Find;
		
			return InternalFind(e);
		}

		/// <internalonly/>
		[Syncfusion.Documentation.DocumentationExclude()]
		protected virtual object InternalFind(GridFindReplaceEventArgs find)
		{
			bool searchUp = (find.Options & GridFindTextOptions.SearchUp) != 0;

			int rowIndex, colIndex;
			if (!Grid.CurrentCell.GetCurrentCell(out rowIndex, out colIndex))
				return null;

			Grid.CurrentCell.ConfirmChanges();

			try
			{
				GridRangeInfo rgCell = GridRangeInfo.Cell(rowIndex, colIndex);
				GridRangeInfo found = null;
				GridRangeInfo selRange = GridRangeInfo.Empty;
				GridRangeInfo wholeRange = Grid.GridCellsRange;

				if ((find.Options & GridFindTextOptions.SelectionOnly) != GridFindTextOptions.None)
					selRange = Grid.Selections.Ranges.GetRangesContaining(rgCell).ActiveRange;

				else if ((find.Options & GridFindTextOptions.ColumnOnly) != GridFindTextOptions.None)
					selRange = GridRangeInfo.Col(colIndex);

				else if ((find.Options & GridFindTextOptions.WholeTable) != GridFindTextOptions.None)
					selRange = GridRangeInfo.Table();

				if (!selRange.IsEmpty)
				{
					selRange = selRange.ExpandRange(wholeRange.Top, wholeRange.Left, wholeRange.Bottom, wholeRange.Right);

					GridRangeInfo rgStart = GridRangeInfo.Cell(selRange.Top, selRange.Left);
					GridRangeInfo rgEnd = GridRangeInfo.Cell(selRange.Bottom, selRange.Right);

					useFirstRowCol = false;
					if (!searchUp)
					{
						found = FindInRange(find, selRange, rgCell, rgEnd);
						if (found == null)
						{
							useFirstRowCol = true;
							found = FindInRange(find, selRange, rgStart, rgCell);
						}
					}
					else
					{
						found = FindInRange(find, selRange, rgCell, rgStart);
						if (found == null)
						{
							useFirstRowCol = true;
							found = FindInRange(find, selRange, rgEnd, rgCell);
						}
					}
					useFirstRowCol = false;
				}

				return found;
			}
			finally
			{
				Grid.CurrentCell.Refresh();
			}
		}

		bool useFirstRowCol = false;

		/// <internalonly/>
		[Syncfusion.Documentation.DocumentationExclude()]
		protected virtual GridRangeInfo FindInRange(GridFindReplaceEventArgs find, GridRangeInfo selRange, GridRangeInfo start, GridRangeInfo end)
		{
			bool searchUp = (find.Options & GridFindTextOptions.SearchUp) != 0;

			using (OperationFeedback feedback = new OperationFeedback(Grid.Model))
			{
				// Return GridRangeInfo if found, otherwise NULL.
				int rowIndex = start.Top;
				int colIndex = start.Left;
				useFirstRowCol |= (op != Operation.Find);
				while (useFirstRowCol || GetNextCell(selRange, ref rowIndex, ref colIndex, true, searchUp))
				{
					useFirstRowCol = false;
					GridStyleInfo style = Grid.Model[rowIndex, colIndex];

					if(style.CellType  == "Header")//Check for header celltype...  
						return null;
					
					GridCellRendererBase renderer = Grid.CellRenderers[style.CellType];
					// TODO: Match case, Match whole cell // set curpos etc necessary?
					bool found = false;
					switch (op)
					{
						case Operation.Find:
							found = renderer.FindText(find.FindString, rowIndex, colIndex, find.Options, true);
							break;
						case Operation.Replace:
							bool flag = true;
							if(style.CellValueType == typeof(System.Int32))
							{
								try
								{
									System.Int32.Parse(find.ReplaceString);
								}
								catch
								{
									flag = false;
								}
							}
							if(flag)
								found = renderer.ReplaceText(find.FindString, find.ReplaceString, rowIndex, colIndex, find.Options, true);
							break;
						case Operation.ReplaceAll:
						    flag = true;
							if(style.CellValueType == typeof(System.Int32))
							{
								try
								{
									System.Int32.Parse(find.ReplaceString);
								}
								catch
								{
									flag = false;
								}
							}
							if(flag)
							replaceAllFound |= renderer.ReplaceText(find.FindString, find.ReplaceString, rowIndex, colIndex, find.Options, false);
							break;
					}
					if (found)
					{
						Grid.CurrentCell.ScrollInView(GridScrollCurrentCellReason.FindText);
						return GridRangeInfo.Cell(rowIndex, colIndex);
					}

					if (rowIndex == end.Top && colIndex == end.Left)
						return null;

					if (feedback.ShouldCancel)
						return null;
				}
				return null;
			}
		}

		/// <internalonly/>
		[Syncfusion.Documentation.DocumentationExclude()]
		static public bool GetNextCell(GridRangeInfo rg, ref int nRow, ref int nCol, bool bSortByRow, bool searchUp)
		{
			if (!searchUp)
			{
				if (bSortByRow && ++nCol > rg.Right)
				{
					if (++nRow > rg.Bottom)
						nRow = nCol = 0;
					else
						nCol = rg.Left;
				}
				else if (!bSortByRow && ++nRow > rg.Bottom)
				{
					if (++nCol > rg.Right)
						nRow = nCol = 0;
					else
						nRow = rg.Top;
				}

				return nRow > 0 || nCol > 0;
			}
			else
			{
				if (bSortByRow && --nCol < rg.Left)
				{
					if (--nRow < rg.Top)
					{ 
						nRow = rg.Bottom; nCol = rg.Right; 
					}
					else
						nCol = rg.Right;
				}
				else if (!bSortByRow && --nRow < rg.Top)
				{
					if (--nCol < rg.Left)
					{ 
						nRow = rg.Bottom; nCol = rg.Right; 
					}
					else
						nRow = rg.Bottom;
				}

				return nRow < rg.Bottom || nCol < rg.Right;
			}
		}


		/// <summary>
		/// Returns the screen bounds for the selection object. The find and replace dialog
		/// calls this method to ensure that it does not hide the current selected text after
		/// a succesful find operation.
		/// </summary>
		/// <param name="lc">The location object, typically a <see cref="GridRangeInfo"/>.</param>
		/// <returns>The bounds in screen coordinates.</returns>
		public virtual Rectangle GetScreenRect(object lc)
		{
			GridRangeInfo rg = lc as GridRangeInfo;
			if (lc != null)
			{	
				Rectangle r = Grid.RangeInfoToRectangle(rg);
				return Grid.RectangleToScreen(r);
			}
			return Rectangle.Empty;
		}
		//		public bool HasSelection 
		//		{ 
		//			get
		//			{
		//				return Grid.CurrentCell.HasCurrentCell;
		//			}
		//		}
		//		public string SelectedText 
		//		{ 
		//			get
		//			{
		//				if (!Grid.CurrentCell.HasCurrentCell)
		//					return "";
		//
		//				GridCellRendererBase renderer = Grid.CurrentCell.Renderer;
		//				string text;
		//				if (renderer.GetSelectedText(out text))
		//					return text;
		//				return ""; // selected text in current cell
		//			}
		//		}
		/// <summary>
		/// Replaces the text with the specified criteria.
		/// </summary>
		/// <param name="e">Text and search criteria.</param>
		/// <returns>The location info (typically a <see cref="GridRangeInfo"/>) or NULL if not found.</returns>
		public virtual object Replace(GridFindReplaceEventArgs e)
		{
			op = Operation.Replace;
			return InternalFind(e);
		}
		/// <summary>
		/// Replace the text in all cells with the specified criteria.
		/// </summary>
		/// <param name="e">Text and search criteria.</param>
		public virtual void ReplaceAll(GridFindReplaceEventArgs e)
		{
			replaceAllFound = false;
			op = Operation.ReplaceAll;
			InternalFind(e);
		}
		/// <summary>
		/// Displays a warning message.
		/// </summary>
		/// <param name="msg">The message text.</param>
		/// <param name="category">The dialog title.</param>
		public virtual void ShowInfoMessage(string msg, string category)
		{
			MessageBox.Show(Grid, msg, category);
		}
	}

}
